@@ -41,13 +41,18 @@ __author__ = "Mathieu Fenniak"
 
 
 def connect(
-        user, host='localhost', unix_sock=None, port=5432, database=None,
-        password=None, ssl=None, timeout=None, application_name=None,
-        max_prepared_statements=1000, tcp_keepalive=True):
+        user, host='localhost', database=None, port=5432, password=None,
+        source_address=None, unix_sock=None, ssl_context=None, timeout=None,
+        max_prepared_statements=1000, tcp_keepalive=True,
+        application_name=None, replication=None):
 
     return Connection(
-        user, host, unix_sock, port, database, password, ssl, timeout,
-        application_name, max_prepared_statements, tcp_keepalive)
+        user, host=host, database=database, port=port, password=password,
+        source_address=source_address, unix_sock=unix_sock,
+        ssl_context=ssl_context, timeout=timeout,
+        max_prepared_statements=max_prepared_statements,
+        tcp_keepalive=tcp_keepalive, application_name=application_name,
+        replication=replication)
 
 
 apilevel = "2.0"
@@ -69,8 +74,6 @@ This property is part of the `DBAPI 2.0 specification
 
 paramstyle = 'format'
 
-max_prepared_statements = 1000
-
 # I have no idea what this would be used for by a client app.  Should it be
 # TEXT, VARCHAR, CHAR?  It will only compare against row_description's
 # type_code if it is this one type.  It is the varchar type oid for now, this
