@@ -785,12 +785,16 @@ class Cursor():
         <http://www.python.org/dev/peps/pep-0249/>`_.
     """
 
-    def __init__(self, connection):
+    def __init__(self, connection, paramstyle=None):
         self._c = connection
         self.arraysize = 1
         self.ps = None
         self._row_count = -1
         self._cached_rows = deque()
+        if paramstyle is None:
+            self.paramstyle = pg8000.paramstyle
+        else:
+            self.paramstyle = paramstyle
 
     def __enter__(self):
         return self
@@ -1023,37 +1027,6 @@ TERMINATE = b'X'
 CLOSE = b'C'
 
 
-def _establish_ssl(_socket, ssl_params):
-    if not isinstance(ssl_params, dict):
-        ssl_params = {}
-
-    try:
-        import ssl as sslmodule
-
-        keyfile = ssl_params.get('keyfile')
-        certfile = ssl_params.get('certfile')
-        ca_certs = ssl_params.get('ca_certs')
-        if ca_certs is None:
-            verify_mode = sslmodule.CERT_NONE
-        else:
-            verify_mode = sslmodule.CERT_REQUIRED
-
-        # Int32(8) - Message length, including self.
-        # Int32(80877103) - The SSL request code.
-        _socket.sendall(ii_pack(8, 80877103))
-        resp = _socket.recv(1)
-        if resp == b'S':
-            return sslmodule.wrap_socket(
-                _socket, keyfile=keyfile, certfile=certfile,
-                cert_reqs=verify_mode, ca_certs=ca_certs)
-        else:
-            raise InterfaceError("Server refuses SSL")
-    except ImportError:
-        raise InterfaceError(
-            "SSL required but ssl module not available in "
-            "this python installation")
-
-
 def create_message(code, data=b''):
     return code + i_pack(len(data) + 4) + data
 
@@ -1118,8 +1091,10 @@ class Connection():
         return error
 
     def __init__(
-            self, user, host, unix_sock, port, database, password, ssl,
-            timeout, application_name, max_prepared_statements, tcp_keepalive):
+            self, user, host='localhost', database=None, port=5432,
+            password=None, source_address=None, unix_sock=None,
+            ssl_context=None, timeout=None, max_prepared_statements=1000,
+            tcp_keepalive=True, application_name=None, replication=None):
         self._client_encoding = "utf8"
         self._commands_with_count = (
             b"INSERT", b"DELETE", b"UPDATE", b"MOVE", b"FETCH", b"COPY",
@@ -1128,15 +1103,30 @@ class Connection():
         self.notices = deque(maxlen=100)
         self.parameter_statuses = deque(maxlen=100)
         self.max_prepared_statements = int(max_prepared_statements)
+        self._run_cursor = Cursor(self, paramstyle='named')
 
         if user is None:
             raise InterfaceError(
                 "The 'user' connection parameter cannot be None")
 
-        if isinstance(user, str):
-            self.user = user.encode('utf8')
-        else:
-            self.user = user
+        init_params = {
+            'user': user,
+            'database': database,
+            'application_name': application_name,
+            'replication': replication
+        }
+
+        for k, v in tuple(init_params.items()):
+            if isinstance(v, str):
+                init_params[k] = v.encode('utf8')
+            elif v is None:
+                del init_params[k]
+            elif not isinstance(v, (bytes, bytearray)):
+                raise InterfaceError(
+                    "The parameter " + k + " can't be of type " +
+                    str(type(v)) + ".")
+
+        self.user = init_params['user']
 
         if isinstance(password, str):
             self.password = password.encode('utf8')
@@ -1151,6 +1141,8 @@ class Connection():
         try:
             if unix_sock is None and host is not None:
                 self._usock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+                if source_address is not None:
+                    self._usock.bind((source_address, 0))
             elif unix_sock is not None:
                 if not hasattr(socket, "AF_UNIX"):
                     raise InterfaceError(
@@ -1168,8 +1160,26 @@ class Connection():
             elif unix_sock is not None:
                 self._usock.connect(unix_sock)
 
-            if ssl:
-                self._usock = _establish_ssl(self._usock, ssl)
+            if ssl_context is not None:
+                try:
+                    import ssl
+
+                    if ssl_context is True:
+                        ssl_context = ssl.create_default_context()
+
+                    # Int32(8) - Message length, including self.
+                    # Int32(80877103) - The SSL request code.
+                    self._usock.sendall(ii_pack(8, 80877103))
+                    resp = self._usock.recv(1)
+                    if resp != b'S':
+                        raise InterfaceError("Server refuses SSL")
+
+                    self._usock = ssl_context.wrap_socket(
+                        self._usock, server_hostname=host)
+                except ImportError:
+                    raise InterfaceError(
+                        "SSL required but ssl module not available in "
+                        "this python installation")
 
             self._sock = self._usock.makefile(mode="rwb")
             if tcp_keepalive:
@@ -1284,6 +1294,17 @@ class Connection():
         def numeric_out(d):
             return str(d).encode(self._client_encoding)
 
+        def inet_out(v):
+            return str(v).encode(self._client_encoding)
+
+        def inet_in(data, offset, length):
+            inet_str = data[offset: offset + length].decode(
+                self._client_encoding)
+            if '/' in inet_str:
+                return ip_network(inet_str, False)
+            else:
+                return ip_address(inet_str)
+
         self.pg_types = defaultdict(
             lambda: (FC_TEXT, text_recv), {
                 16: (FC_BINARY, bool_recv),  # boolean
@@ -1301,6 +1322,7 @@ class Connection():
                 701: (FC_BINARY, float8_recv),  # float8
                 705: (FC_BINARY, text_recv),  # unknown
                 829: (FC_TEXT, text_recv),  # MACADDR type
+                869: (FC_TEXT, inet_in),  # inet
                 1000: (FC_BINARY, array_recv),  # BOOL[]
                 1003: (FC_BINARY, array_recv),  # NAME[]
                 1005: (FC_BINARY, array_recv),  # INT2[]
@@ -1324,7 +1346,8 @@ class Connection():
                 2275: (FC_BINARY, text_recv),  # cstring
                 2950: (FC_BINARY, uuid_recv),  # uuid
                 3802: (FC_TEXT, json_in),  # jsonb
-            })
+            }
+        )
 
         self.py_types = {
             type(None): (-1, FC_BINARY, null_send),  # null
@@ -1348,34 +1371,22 @@ class Connection():
             Interval: (1186, FC_BINARY, interval_send_integer),
             Decimal: (1700, FC_TEXT, numeric_out),  # Decimal
             PGTsvector: (3614, FC_TEXT, text_out),
-            UUID: (2950, FC_BINARY, uuid_send)}  # uuid
+            UUID: (2950, FC_BINARY, uuid_send),  # uuid
+            bytes: (17, FC_BINARY, bytea_send),  # bytea
+            str: (705, FC_TEXT, text_out),  # unknown
+            enum.Enum: (705, FC_TEXT, enum_out),
+            IPv4Address: (869, FC_TEXT, inet_out),  # inet
+            IPv6Address: (869, FC_TEXT, inet_out),  # inet
+            IPv4Network: (869, FC_TEXT, inet_out),  # inet
+            IPv6Network: (869, FC_TEXT, inet_out)  # inet
+        }
 
         self.inspect_funcs = {
             Datetime: self.inspect_datetime,
             list: self.array_inspect,
             tuple: self.array_inspect,
-            int: self.inspect_int}
-
-        self.py_types[bytes] = (17, FC_BINARY, bytea_send)  # bytea
-        self.py_types[str] = (705, FC_TEXT, text_out)  # unknown
-        self.py_types[enum.Enum] = (705, FC_TEXT, enum_out)
-
-        def inet_out(v):
-            return str(v).encode(self._client_encoding)
-
-        def inet_in(data, offset, length):
-            inet_str = data[offset: offset + length].decode(
-                self._client_encoding)
-            if '/' in inet_str:
-                return ip_network(inet_str, False)
-            else:
-                return ip_address(inet_str)
-
-        self.py_types[IPv4Address] = (869, FC_TEXT, inet_out)  # inet
-        self.py_types[IPv6Address] = (869, FC_TEXT, inet_out)  # inet
-        self.py_types[IPv4Network] = (869, FC_TEXT, inet_out)  # inet
-        self.py_types[IPv6Network] = (869, FC_TEXT, inet_out)  # inet
-        self.pg_types[869] = (FC_TEXT, inet_in)  # inet
+            int: self.inspect_int
+        }
 
         self.message_types = {
             NOTICE_RESPONSE: self.handle_NOTICE_RESPONSE,
@@ -1406,17 +1417,10 @@ class Connection():
         #   String - A parameter name (user, database, or options)
         #   String - Parameter value
         protocol = 196608
-        val = bytearray(
-            i_pack(protocol) + b"user\x00" + self.user + NULL_BYTE)
-        if database is not None:
-            if isinstance(database, str):
-                database = database.encode('utf8')
-            val.extend(b"database\x00" + database + NULL_BYTE)
-        if application_name is not None:
-            if isinstance(application_name, str):
-                application_name = application_name.encode('utf8')
-            val.extend(
-                b"application_name\x00" + application_name + NULL_BYTE)
+        val = bytearray(i_pack(protocol))
+
+        for k, v in init_params.items():
+            val.extend(k.encode('ascii') + NULL_BYTE + v + NULL_BYTE)
         val.append(0)
         self._write(i_pack(len(val) + 4))
         self._write(val)
@@ -1543,6 +1547,14 @@ class Connection():
         """
         return Cursor(self)
 
+    @property
+    def description(self):
+        return self._run_cursor._getDescription()
+
+    def run(self, sql, stream=None, **params):
+        self._run_cursor.execute(sql, params, stream=stream)
+        return tuple(self._run_cursor._cached_rows)
+
     def commit(self):
         """Commits the current database transaction.
 
@@ -1690,6 +1702,7 @@ class Connection():
             return self.py_types[23]
         if min_int8 < value < max_int8:
             return self.py_types[20]
+        return self.py_types[Decimal]
 
     def make_params(self, values):
         params = []
@@ -1749,15 +1762,14 @@ class Connection():
         if vals is None:
             vals = ()
 
-        paramstyle = pg8000.paramstyle
         pid = getpid()
         try:
-            cache = self._caches[paramstyle][pid]
+            cache = self._caches[cursor.paramstyle][pid]
         except KeyError:
             try:
-                param_cache = self._caches[paramstyle]
+                param_cache = self._caches[cursor.paramstyle]
             except KeyError:
-                param_cache = self._caches[paramstyle] = {}
+                param_cache = self._caches[cursor.paramstyle] = {}
 
             try:
                 cache = param_cache[pid]
@@ -1768,7 +1780,7 @@ class Connection():
             statement, make_args = cache['statement'][operation]
         except KeyError:
             statement, make_args = cache['statement'][operation] = \
-                convert_paramstyle(paramstyle, operation)
+                convert_paramstyle(cursor.paramstyle, operation)
 
         args = make_args(vals)
         params = self.make_params(args)
